@@ -12,12 +12,12 @@ to_upper_case(std::string& s)
     // C(++), I hate you...
     int (*touppercase)(int) = std::toupper;
     /****************************/
-    std::transform(s.begin(),s.end(),s.begin(),touppercase);
+    std::transform(s.begin(), s.end(), s.begin(), touppercase);
 }
 
 
 void
-format(
+to_xml(
     std::string& in
 )
 {
@@ -28,6 +28,22 @@ format(
         in.replace(pos, 1, "&amp;");
         pos += 5;
     }
+    
+    pos = 0;
+
+    while ((pos = in.find("<", pos)) != std::string::npos)
+    {
+        in.replace(pos, 1, "&lt;");
+        pos += 4;
+    }
+    
+    pos = 0;
+
+    while ((pos = in.find(">", pos)) != std::string::npos)
+    {
+        in.replace(pos, 1, "&gt;");
+        pos += 4;
+    }
 }
 
 }
