@@ -152,16 +152,16 @@ read_metadata(
         case GraphIOFileSection::VERTEX_ATTRIBUTES:
         {
             size_t from_idx = 0;
-            core::Attribute vertex_att = read_attr_def(fields, from_idx, csv.row_num());
-            meta.vertex_attributes.push_back(vertex_att);
+            auto vertex_att = read_attr_def(fields, from_idx, csv.row_num());
+            meta.vertex_attributes.push_back(std::move(vertex_att));
             break;
         }
 
         case GraphIOFileSection::EDGE_ATTRIBUTES:
         {
             size_t from_idx = 0;
-            core::Attribute edge_att = read_attr_def(fields, from_idx, csv.row_num());
-            meta.edge_attributes.push_back(edge_att);
+            auto edge_att = read_attr_def(fields, from_idx, csv.row_num());
+            meta.edge_attributes.push_back(std::move(edge_att));
             break;
         }
 
